@@ -28,8 +28,9 @@ defined('MOODLE_INTERNAL') || die();
 
 require_once("$CFG->libdir/filelib.php");
 
-use H5PCore;
-use H5PFrameworkInterface;
+use Moodle\H5PCore;
+use Moodle\H5PFrameworkInterface;
+use Moodle\H5PHubEndpoints;
 use stdClass;
 use moodle_url;
 use core_h5p\local\library\autoloader;
@@ -41,7 +42,7 @@ use core_h5p\local\library\autoloader;
  * @copyright  2019 Sara Arjona <sara@moodle.com>
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-class core extends \H5PCore {
+class core extends H5PCore {
 
     /** @var array The array containing all the present libraries */
     protected $libraries;
@@ -50,7 +51,7 @@ class core extends \H5PCore {
      * Constructor for core_h5p/core.
      *
      * @param H5PFrameworkInterface $framework The frameworks implementation of the H5PFrameworkInterface
-     * @param string|\H5PFileStorage $path The H5P file storage directory or class
+     * @param string|H5PFileStorage $path The H5P file storage directory or class
      * @param string $url The URL to the file storage directory
      * @param string $language The language code. Defaults to english
      * @param boolean $export Whether export is enabled
@@ -213,7 +214,7 @@ class core extends \H5PCore {
             if ($shoulddownload) {
                 $installed['id'] = $this->fetch_content_type($library);
                 if ($installed['id']) {
-                    $installed['name'] = \H5PCore::libraryToString($library);
+                    $installed['name'] = H5PCore::libraryToString($library);
                     $typesinstalled[] = $installed;
                 }
             }
@@ -237,8 +238,22 @@ class core extends \H5PCore {
 
         $factory = new factory();
 
-        // Download the latest content type from the H5P official repository.
         $fs = get_file_storage();
+
+        // Delete any existing file, if it was not deleted during a previous download.
+        $existing = $fs->get_file(
+            (\context_system::instance())->id,
+            'core_h5p',
+            'library_sources',
+            0,
+            '/',
+            $library['machineName']
+        );
+        if ($existing) {
+            $existing->delete();
+        }
+
+        // Download the latest content type from the H5P official repository.
         $file = $fs->create_file_from_url(
             (object) [
                 'component' => 'core_h5p',
@@ -292,9 +307,9 @@ class core extends \H5PCore {
      */
     public function get_api_endpoint(?string $library = null, string $endpoint = 'content'): moodle_url {
         if ($endpoint == 'site') {
-            $h5purl = \H5PHubEndpoints::createURL(\H5PHubEndpoints::SITES );
+            $h5purl = H5PHubEndpoints::createURL(H5PHubEndpoints::SITES );
         } else if ($endpoint == 'content') {
-            $h5purl = \H5PHubEndpoints::createURL(\H5PHubEndpoints::CONTENT_TYPES ) . $library;
+            $h5purl = H5PHubEndpoints::createURL(H5PHubEndpoints::CONTENT_TYPES ) . $library;
         }
 
         return new moodle_url($h5purl);
