@@ -333,7 +333,7 @@ class helper {
         // When there is a logged in user, her information will be passed to the player. It will be used for tracking.
         $usersettings = [];
         if (isloggedin()) {
-            $usersettings['name'] = fullname($USER, has_capability('moodle/site:viewfullnames', $systemcontext));
+            $usersettings['name'] = $USER->username;
             $usersettings['id'] = $USER->id;
         }
         $settings = array(
@@ -348,7 +348,7 @@ class helper {
             'user' => $usersettings,
             'hubIsEnabled' => true,
             'reportingIsEnabled' => false,
-            'crossorigin' => null,
+            'crossorigin' => !empty($CFG->h5pcrossorigin) ? $CFG->h5pcrossorigin : null,
             'libraryConfig' => $core->h5pF->getLibraryConfig(),
             'pluginCacheBuster' => self::get_cache_buster(),
             'libraryUrl' => autoloader::get_h5p_core_library_url('js')->out(),
