@@ -28,7 +28,6 @@
 require_once(__DIR__ . '/../../behat/behat_base.php');
 
 use Behat\Gherkin\Node\TableNode;
-use Behat\Mink\Element\NodeElement;
 use Behat\Mink\Exception\DriverException;
 use Behat\Mink\Exception\ElementNotFoundException;
 use Behat\Mink\Exception\ExpectationException;
@@ -148,7 +147,7 @@ class behat_general extends behat_base {
 
         } else if (!empty($url)) {
             // We redirect directly as we can not wait for an automatic redirection.
-            $this->getSession()->getDriver()->getClient()->request('GET', $url);
+            $this->getSession()->getDriver()->getClient()->request('get', $url);
 
         } else {
             // Reload the page if no URL was provided.
@@ -188,22 +187,6 @@ class behat_general extends behat_base {
         );
     }
 
-    /**
-     * Switches to a second window.
-     *
-     * @Given /^I switch to a second window$/
-     * @throws DriverException If there aren't exactly 2 windows open.
-     */
-    public function switch_to_second_window() {
-        $names = $this->getSession()->getWindowNames();
-
-        if (count($names) !== 2) {
-            throw new DriverException('Expected to see 2 windows open, found ' . count($names));
-        }
-
-        $this->getSession()->switchToWindow($names[1]);
-    }
-
     /**
      * Switches to the main Moodle frame.
      *
@@ -369,6 +352,22 @@ class behat_general extends behat_base {
         $node->mouseOver();
     }
 
+    /**
+     * Generic mouse over action. Mouse over a element of the specified type.
+     *
+     * @When /^I hover over the "(?P<element_string>(?:[^"]|\\")*)" "(?P<selector_string>[^"]*) in the "(?P<container_element_string>(?:[^"]|\\")*)" "(?P<container_selector_string>[^"]*)"$/
+     * @param string $element Element we look for
+     * @param string $selectortype The type of what we look for
+     * @param string $containerelement Element we look for
+     * @param string $containerselectortype The type of what we look for
+     */
+    public function i_hover_in_the(string $element, $selectortype, string $containerelement, $containerselectortype): void {
+        // Gets the node based on the requested selector type and locator.
+        $node = $this->get_node_in_container($selectortype, $element, $containerselectortype, $containerselectortype);
+        $this->execute_js_on_node($node, '{{ELEMENT}}.scrollIntoView();');
+        $node->mouseOver();
+    }
+
     /**
      * Generic click action. Click on the element of the specified type.
      *
@@ -1027,6 +1026,45 @@ EOF;
         throw new ExpectationException("The '{$element}' '{$selectortype}' exists in the current page", $this->getSession());
     }
 
+    /**
+     * Ensure that edit mode is (not) available on the current page.
+     *
+     * @Then edit mode should be available on the current page
+     * @Then edit mode should :not be available on the current page
+     * @param bool $not
+     */
+    public function edit_mode_should_be_available(bool $not = false): void {
+        $isavailable = $this->is_edit_mode_available();
+        $shouldbeavailable = empty($not);
+
+        if ($isavailable && !$shouldbeavailable) {
+            throw new ExpectationException("Edit mode is available and should not be", $this->getSession());
+        } else if ($shouldbeavailable && !$isavailable) {
+            throw new ExpectationException("Edit mode is not available and should be", $this->getSession());
+        }
+    }
+
+    /**
+     * Check whether edit mode is available on the current page.
+     *
+     * @return bool
+     */
+    public function is_edit_mode_available(): bool {
+        // If the course is already in editing mode then it will have the class 'editing' on the body.
+        // This is a 'cheap' way of telling if the course is in editing mode and therefore if edit mode is available.
+        $body = $this->find('css', 'body');
+        if ($body->hasClass('editing')) {
+            return true;
+        }
+
+        try {
+            $this->find('field', get_string('editmode'), false, false, 0);
+            return true;
+        } catch (ElementNotFoundException $e) {
+            return false;
+        }
+    }
+
     /**
      * This step triggers cron like a user would do going to admin/cron.php.
      *
@@ -1163,12 +1201,19 @@ EOF;
      * @throws ElementNotFoundException Thrown by behat_base::find
      * @param string $element The locator of the specified selector
      * @param string $selectortype The selector type
-     * @param NodeElement|string $containerelement The locator of the container selector
-     * @param string $containerselectortype The container selector type
+     * @param string $containerelement The container selector type
+     * @param string $containerselectortype The container locator
      */
     public function should_exist_in_the($element, $selectortype, $containerelement, $containerselectortype) {
-        // Will throw an ElementNotFoundException if it does not exist.
-        $this->get_node_in_container($selectortype, $element, $containerselectortype, $containerelement);
+        // Get the container node.
+        $containernode = $this->find($containerselectortype, $containerelement);
+
+        // Specific exception giving info about where can't we find the element.
+        $locatorexceptionmsg = "{$element} in the {$containerelement} {$containerselectortype}";
+        $exception = new ElementNotFoundException($this->getSession(), $selectortype, null, $locatorexceptionmsg);
+
+        // Looks for the requested node inside the container node.
+        $this->find($selectortype, $element, $exception, $containernode);
     }
 
     /**
@@ -1180,8 +1225,8 @@ EOF;
      * @throws ExpectationException
      * @param string $element The locator of the specified selector
      * @param string $selectortype The selector type
-     * @param NodeElement|string $containerelement The locator of the container selector
-     * @param string $containerselectortype The container selector type
+     * @param string $containerelement The container selector type
+     * @param string $containerselectortype The container locator
      */
     public function should_not_exist_in_the($element, $selectortype, $containerelement, $containerselectortype) {
         // Get the container node.
@@ -1198,25 +1243,31 @@ EOF;
         }
 
         // The element was found and should not have been. Throw an exception.
-        $elementdescription = $this->get_selector_description($selectortype, $element);
-        $containerdescription = $this->get_selector_description($containerselectortype, $containerelement);
         throw new ExpectationException(
-            "The {$elementdescription} exists in the {$containerdescription}",
+            "The '{$element}' '{$selectortype}' exists in the '{$containerelement}' '{$containerselectortype}'",
             $this->getSession()
         );
     }
 
     /**
-     * Change browser window size small: 640x480, medium: 1024x768, large: 2560x1600, custom: widthxheight
+     * Change browser window size
+     *
+     * Allowed sizes:
+     * - mobile: 425x750
+     * - tablet: 768x1024
+     * - small: 1024x768
+     * - medium: 1366x768
+     * - large: 2560x1600
+     * - custom: widthxheight
      *
      * Example: I change window size to "small" or I change window size to "1024x768"
      * or I change viewport size to "800x600". The viewport option is useful to guarantee that the
      * browser window has same viewport size even when you run Behat on multiple operating systems.
      *
      * @throws ExpectationException
-     * @Then /^I change (window|viewport) size to "(small|medium|large|\d+x\d+)"$/
-     * @Then /^I change the (window|viewport) size to "(small|medium|large|\d+x\d+)"$/
-     * @param string $windowsize size of the window (small|medium|large|wxh).
+     * @Then /^I change (window|viewport) size to "(mobile|tablet|small|medium|large|\d+x\d+)"$/
+     * @Then /^I change the (window|viewport) size to "(mobile|tablet|small|medium|large|\d+x\d+)"$/
+     * @param string $windowsize size of the window (mobile|tablet|small|medium|large|wxh).
      */
     public function i_change_window_size_to($windowviewport, $windowsize) {
         $this->resize_window($windowsize, $windowviewport === 'viewport');
@@ -1725,17 +1776,7 @@ EOF;
      * @param   string $keys The key, or list of keys, to type
      */
     public function i_type(string $keys): void {
-        // Certain keys, such as the newline character, must be converted to the appropriate character code.
-        // Without this, keys will behave differently depending on the browser.
-        $keylist = array_map(function($key): string {
-            switch ($key) {
-                case "\n":
-                    return behat_keys::ENTER;
-                default:
-                    return $key;
-            }
-        }, str_split($keys));
-        behat_base::type_keys($this->getSession(), $keylist);
+        behat_base::type_keys($this->getSession(), str_split($keys));
     }
 
     /**
@@ -1853,6 +1894,9 @@ EOF;
             case 'SPACE':
                 $keys[] = behat_keys::SPACE;
                 break;
+            case 'MULTIPLY':
+                $keys[] = behat_keys::MULTIPLY;
+                break;
             default:
                 // You can enter a single ASCII character (e.g. a letter) to directly type that key.
                 if (strlen($key) === 1) {
@@ -1944,21 +1988,6 @@ EOF;
         throw new \Moodle\BehatExtension\Exception\SkippedException();
     }
 
-    /**
-     * Checks if given plugin is installed, and skips the current scenario if not.
-     *
-     * @Given the :plugin plugin is installed
-     * @param string $plugin frankenstyle plugin name, e.g. 'filter_embedquestion'.
-     * @throws \Moodle\BehatExtension\Exception\SkippedException
-     */
-    public function plugin_is_installed(string $plugin): void {
-        $path = core_component::get_component_directory($plugin);
-        if (!is_readable($path . '/version.php')) {
-            throw new \Moodle\BehatExtension\Exception\SkippedException(
-                    'Skipping this scenario because the ' . $plugin . ' is not installed.');
-        }
-    }
-
     /**
      * Checks focus is with the given element.
      *
@@ -2123,102 +2152,27 @@ EOF;
     }
 
     /**
-     * Set the default text editor to the named text editor.
+     * Click on a dynamic tab to load its content
      *
-     * @Given the default editor is set to :editor
-     * @param string $editor
-     * @throws ExpectationException If the specified editor is not available.
+     * @Given /^I click on the "(?P<tab_string>(?:[^"]|\\")*)" dynamic tab$/
+     *
+     * @param string $tabname
      */
-    public function the_default_editor_is_set_to(string $editor): void {
-        global $CFG;
-
-        // Check if the provided editor is available.
-        if (!array_key_exists($editor, editors_get_available())) {
-            throw new ExpectationException(
-                "Unable to set the editor to {$editor} as it is not installed. The available editors are: " .
-                    implode(', ', array_keys(editors_get_available())),
-                $this->getSession()
-            );
-        }
-
-        // Make the provided editor the default one in $CFG->texteditors by
-        // moving it to the first [editor],atto,tiny,tinymce,textarea on the list.
-        $list = explode(',', $CFG->texteditors);
-        array_unshift($list, $editor);
-        $list = array_unique($list);
-
-        // Set the list new list of editors.
-        set_config('texteditors', implode(',', $list));
+    public function i_click_on_the_dynamic_tab(string $tabname): void {
+        $xpath = "//*[@id='dynamictabs-tabs'][descendant::a[contains(text(), '" . $this->escape($tabname) . "')]]";
+        $this->execute('behat_general::i_click_on_in_the',
+            [$tabname, 'link', $xpath, 'xpath_element']);
     }
 
     /**
-     * Allow to check for minimal Moodle version.
+     * Enable an specific plugin.
      *
-     * @Given the site is running Moodle version :minversion or higher
-     * @param string $minversion The minimum version of Moodle required (inclusive).
+     * @When /^I enable "(?P<plugin_string>(?:[^"]|\\")*)" "(?P<plugintype_string>[^"]*)" plugin$/
+     * @param string $plugin Plugin we look for
+     * @param string $plugintype The type of the plugin
      */
-    public function the_site_is_running_moodle_version_or_higher(string $minversion): void {
-        global $CFG;
-        require_once($CFG->libdir . '/environmentlib.php');
-
-        $currentversion = normalize_version(get_config('', 'release'));
-
-        if (version_compare($currentversion, $minversion, '<')) {
-            throw new Moodle\BehatExtension\Exception\SkippedException(
-                'Site must be running Moodle version ' . $minversion . ' or higher'
-            );
-        }
-    }
-
-    /**
-     * Allow to check for maximum Moodle version.
-     *
-     * @Given the site is running Moodle version :maxversion or lower
-     * @param string $maxversion The maximum version of Moodle required (inclusive).
-     */
-    public function the_site_is_running_moodle_version_or_lower(string $maxversion): void {
-        global $CFG;
-        require_once($CFG->libdir . '/environmentlib.php');
-
-        $currentversion = normalize_version(get_config('', 'release'));
-
-        if (version_compare($currentversion, $maxversion, '>')) {
-            throw new Moodle\BehatExtension\Exception\SkippedException(
-                'Site must be running Moodle version ' . $maxversion . ' or lower'
-            );
-        }
-    }
-
-    /**
-     * Check that the page title contains a given string.
-     *
-     * @Given the page title should contain ":title"
-     * @param string $title The string that should be present on the page title.
-     */
-    public function the_page_title_should_contain(string $title): void {
-        $session = $this->getSession();
-        if ($this->running_javascript()) {
-            // When running on JS, the page title can be changed via JS, so it's more reliable to get the actual page title via JS.
-            $actualtitle = $session->evaluateScript("return document.title");
-        } else {
-            $titleelement = $session->getPage()->find('css', 'head title');
-            if ($titleelement === null) {
-                // Throw an exception if a page title is not present on the page.
-                throw new ElementNotFoundException(
-                    $this->getSession(),
-                    '<title> element',
-                    'css',
-                    'head title'
-                );
-            }
-            $actualtitle = $titleelement->getText();
-        }
-
-        if (strpos($actualtitle, $title) === false) {
-            throw new ExpectationException(
-                "'$title' was not found from the current page title '$actualtitle'",
-                $session
-            );
-        }
+    public function i_enable_plugin($plugin, $plugintype) {
+        $class = core_plugin_manager::resolve_plugininfo_class($plugintype);
+        $class::enable_plugin($plugin, true);
     }
 }
