@@ -1,5 +1,3 @@
-/* global DIALOGUE_PREFIX, BASE */
-
 /**
  * The generic dialogue class for use in Moodle.
  *
@@ -15,7 +13,13 @@ var DIALOGUE_NAME = 'Moodle dialogue',
     MENUBAR_SELECTOR = '[role=menubar]',
     DOT = '.',
     HAS_ZINDEX = 'moodle-has-zindex',
-    CAN_RECEIVE_FOCUS_SELECTOR = 'input:not([type="hidden"]), a[href], button, textarea, select, [tabindex]';
+    CAN_RECEIVE_FOCUS_SELECTOR = 'input:not([type="hidden"]):not([disabled]):not([tabindex^="-"]),' +
+        'a[href]:not([disabled]):not([tabindex^="-"]),' +
+        'button:not([disabled]):not([tabindex^="-"]),' +
+        'textarea:not([disabled]):not([tabindex^="-"]),' +
+        'select:not([disabled]):not([tabindex^="-"]),' +
+        '[tabindex]:not([disabled]):not([tabindex^="-"])',
+    FORM_SELECTOR = 'form';
 
 /**
  * A re-usable dialogue box with Moodle classes applied.
@@ -32,16 +36,20 @@ DIALOGUE = function(config) {
     // during widget instantiation, however, because we're creating it on the fly (and 'config.srcNode' isn't set yet), care must
     // be taken to add it to the DOM and to properly set the value of 'config.srcNode' before calling the parent constructor.
     // Note: additional classes can be added to this content node by setting the 'additionalBaseClass' config property (a string).
-    var id = 'moodle-dialogue-' + Y.stamp(this); // Can't use this.get('id') as it's not set at this stage.
+    var id = 'moodle-dialogue-' + Y.stamp(this) + '-wrap'; // Can't use this.get('id') as it's not set at this stage.
     config.notificationBase =
-        Y.Node.create('<div class="' + CSS.BASE + '">')
-              .append(Y.Node.create('<div id="' + id + '" role="dialog" ' +
-                                    'aria-labelledby="' + id + '-header-text" class="' + CSS.WRAP + '"></div>')
-              .append(Y.Node.create('<div id="' + id + '-header-text" class="' + CSS.HEADER + ' yui3-widget-hd"></div>'))
-              .append(Y.Node.create('<div class="' + CSS.BODY + ' yui3-widget-bd"></div>'))
-              .append(Y.Node.create('<div class="' + CSS.FOOTER + ' yui3-widget-ft"></div>')));
-    Y.one(document.body).append(config.notificationBase);
+        Y.Node.create('<div class="' + CSS_CLASSES.BASE + '">')
+            .append(Y.Node.create(
+                '<div id="' + id + '" role="dialog" ' +
+                'aria-labelledby="' + id + '-header-text" class="' + CSS_CLASSES.WRAP + '"  aria-live="polite"></div>'
+            )
+            .append(Y.Node.create('<div class="' + CSS_CLASSES.HEADER + ' yui3-widget-hd"></div>'))
+            .append(Y.Node.create('<div class="' + CSS_CLASSES.BODY + ' yui3-widget-bd"></div>'))
+            .append(Y.Node.create('<div class="' + CSS_CLASSES.FOOTER + ' yui3-widget-ft"></div>')));
+    config.attachmentPoint = config.attachmentPoint || document.body;
+    Y.one(config.attachmentPoint).append(config.notificationBase);
     config.srcNode = '#' + id;
+    delete config.buttons; // Don't let anyone pass in buttons as we want to control these during init. addButton can be used later.
     DIALOGUE.superclass.constructor.apply(this, [config]);
 };
 Y.extend(DIALOGUE, Y.Panel, {
@@ -50,7 +58,8 @@ Y.extend(DIALOGUE, Y.Panel, {
     // Orientation change event listener.
     _orientationevent: null,
     _calculatedzindex: false,
-
+    // Current maskNode id
+    _currentMaskNodeId: null,
     /**
      * The original position of the dialogue before it was reposition to
      * avoid browser jumping.
@@ -71,6 +80,20 @@ Y.extend(DIALOGUE, Y.Panel, {
      */
     _hiddenSiblings: null,
 
+    /**
+     * Hide the modal only if it doesn't contain a form.
+     *
+     * @method hideIfNotForm
+     */
+    hideIfNotForm: function() {
+        var bb = this.get('boundingBox'),
+            formElement = bb.one(FORM_SELECTOR);
+
+        if (formElement === null) {
+            this.hide();
+        }
+    },
+
     /**
      * Initialise the dialogue.
      *
@@ -80,10 +103,16 @@ Y.extend(DIALOGUE, Y.Panel, {
         var bb;
 
         if (this.get('closeButton') !== false) {
-            // The buttons constructor does not allow custom attributes
-            this.get('buttons').header[0].setAttribute('title', this.get('closeButtonTitle'));
+            var title = this.get('closeButtonTitle');
+            // The buttons constructor does not allow custom attributes.
+            this.get('buttons').header[0].setAttribute('title', title);
+            this.get('buttons').header[0].setAttribute('aria-label', title);
         }
 
+        this.setStdModContent(Y.WidgetStdMod.HEADER,
+            '<h5 id="' + this.get('id') + '-wrap-header-text">' + this.get('headerContent') + '</h5>',
+            Y.WidgetStdMod.REPLACE);
+
         // Initialise the element cache.
         this._hiddenSiblings = [];
 
@@ -102,6 +131,12 @@ Y.extend(DIALOGUE, Y.Panel, {
             this.plug(Y.M.core.LockScroll);
         }
 
+        // Remove the `focusoutside` listener.
+        // It conflicts with the ARIA focuslock manager which supports both YUI and non-YUI dialogues.
+        this.set('focusOn', Y.Array(this.get('focusOn')).filter(function(node) {
+            return node.eventName !== 'focusoutside';
+        }));
+
         // Workaround upstream YUI bug http://yuilibrary.com/projects/yui3/ticket/2532507
         // and allow setting of z-index in theme.
         bb = this.get('boundingBox');
@@ -112,6 +147,13 @@ Y.extend(DIALOGUE, Y.Panel, {
 
         if (this.get('visible')) {
             this.applyZIndex();
+            this.applyAndTrapFocus();
+            // Only do accessibility hiding for modals because the ARIA spec
+            // says that all ARIA dialogues should be modal.
+            if (this.get('modal')) {
+                // Make this dialogue visible to screen readers.
+                this.setAccessibilityVisible();
+            }
         }
         // Recalculate the zIndex every time the modal is altered.
         this.on('maskShow', this.applyZIndex);
@@ -126,6 +168,13 @@ Y.extend(DIALOGUE, Y.Panel, {
                 this._originalPosition = bb.getXY();
             }
 
+            // Check if maskNode already init click event.
+            var maskNode = this.get('maskNode');
+            if (this._currentMaskNodeId !== maskNode.get('_yuid')) {
+                this._currentMaskNodeId = maskNode.get('_yuid');
+                maskNode.on('click', this.hideIfNotForm, this);
+            }
+
             if (bb.getStyle('position') !== 'fixed') {
                 // If the boundingBox has been positioned in a fixed manner, then it will not position correctly to scrollTop.
                 bb.setStyles({
@@ -155,7 +204,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @method applyZIndex
      */
     applyZIndex: function() {
-        var highestzindex = 1,
+        var highestzindex = 1040,
             zindexvalue = 1,
             bb = this.get('boundingBox'),
             ol = this.get('maskNode'),
@@ -219,7 +268,7 @@ Y.extend(DIALOGUE, Y.Panel, {
     visibilityChanged: function(e) {
         var titlebar, bb;
         if (e.attrName === 'visible') {
-            this.get('maskNode').addClass(CSS.LIGHTBOX);
+            this.get('maskNode').addClass(CSS_CLASSES.LIGHTBOX);
             // Going from visible to hidden.
             if (e.prevVal && !e.newVal) {
                 bb = this.get('boundingBox');
@@ -231,7 +280,10 @@ Y.extend(DIALOGUE, Y.Panel, {
                     this._orientationevent.detach();
                     this._orientationevent = null;
                 }
-                bb.detach('key', this.keyDelegation);
+                require(['core/local/aria/focuslock'], function(FocusLockManager) {
+                    // Untrap focus when the dialogue is hidden.
+                    FocusLockManager.untrapFocus();
+                });
 
                 if (this.get('modal')) {
                     // Hide this dialogue from screen readers.
@@ -246,12 +298,11 @@ Y.extend(DIALOGUE, Y.Panel, {
                 this.makeResponsive();
                 if (!this.shouldResizeFullscreen()) {
                     if (this.get('draggable')) {
-                        titlebar = '#' + this.get('id') + ' .' + CSS.HEADER;
+                        titlebar = '#' + this.get('id') + ' .' + CSS_CLASSES.HEADER;
                         this.plug(Y.Plugin.Drag, {handles: [titlebar]});
                         Y.one(titlebar).setStyle('cursor', 'move');
                     }
                 }
-                this.keyDelegation();
 
                 // Only do accessibility hiding for modals because the ARIA spec
                 // says that all ARIA dialogues should be modal.
@@ -342,35 +393,17 @@ Y.extend(DIALOGUE, Y.Panel, {
                Math.floor(Y.one(document.body).get('winWidth')) < this.get('responsiveWidth');
     },
 
+    _focus: function() {
+        this.focus();
+    },
+
     show: function() {
-        var result = null,
-            header = this.headerNode,
-            content = this.bodyNode,
-            focusSelector = this.get('focusOnShowSelector'),
-            focusNode = null;
-
-        result = DIALOGUE.superclass.show.call(this);
-
+        var result = DIALOGUE.superclass.show.call(this);
         if (!this.get('center') && this._originalPosition) {
             // Restore the dialogue position to it's location before it was moved at show time.
             this.get('boundingBox').setXY(this._originalPosition);
         }
-
-        // Try and find a node to focus on using the focusOnShowSelector attribute.
-        if (focusSelector !== null) {
-            focusNode = this.get('boundingBox').one(focusSelector);
-        }
-        if (!focusNode) {
-            // Fall back to the header or the content if no focus node was found yet.
-            if (header && header !== '') {
-                focusNode = header;
-            } else if (content && content !== '') {
-                focusNode = content;
-            }
-        }
-        if (focusNode) {
-            focusNode.focus();
-        }
+        this.applyAndTrapFocus();
         return result;
     },
 
@@ -396,6 +429,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @method keyDelegation
      */
     keyDelegation: function() {
+        Y.log('The keyDelegation function has been deprecated in favour of the AMD core/local/aria/focuslock module');
         var bb = this.get('boundingBox');
         bb.delegate('key', function(e) {
             var target = e.target;
@@ -492,7 +526,37 @@ Y.extend(DIALOGUE, Y.Panel, {
 
         // Clear the cache. No longer need to store these.
         this._hiddenSiblings = [];
-    }
+    },
+
+    /**
+     * Focuses on the node specified by focusOnShowSelector, or the first focusable node if nothing is specified.
+     * It also traps the focus to the current bounding box.
+     *
+     * @method applyAndTrapFocus
+     */
+    applyAndTrapFocus: function() {
+        var content = this.bodyNode;
+        var focusSelector = this.get('focusOnShowSelector');
+        var focusNode = null;
+
+        // Try and find a node to focus on using the focusOnShowSelector attribute.
+        if (focusSelector !== null) {
+            focusNode = this.get('boundingBox').one(focusSelector);
+        }
+        if (!focusNode) {
+            // Fall back to the first focusable element in the body of the dialogue if no focus node was found yet.
+            if (content && content !== '') {
+                focusNode = content.one(CAN_RECEIVE_FOCUS_SELECTOR);
+            }
+        }
+        require(['core/local/aria/focuslock'], function(FocusLockManager) {
+            // Trap focus to the current bounding box.
+            FocusLockManager.trapFocus(this.get('boundingBox').getDOMNode());
+            if (focusNode) {
+                focusNode.focus();
+            }
+        }.bind(this));
+    },
 }, {
     NAME: DIALOGUE_NAME,
     CSS_PREFIX: DIALOGUE_PREFIX,
@@ -748,9 +812,8 @@ Y.Base.modifyAttrs(DIALOGUE, {
      * @default {}
      */
     buttons: {
-        // Readonly is really important. We don't want to allow users of the plugin to pass in buttons. closeButton handles this.
-        readOnly: true,
         getter: Y.WidgetButtons.prototype._getButtons,
+        setter: Y.WidgetButtons.prototype._setButtons,
         valueFn: function() {
             if (this.get('closeButton') === false) {
                 return null;
