@@ -24,7 +24,9 @@
  */
 
 $string['addcomment'] = 'Add comment';
+$string['addcategory'] = 'Add category';
 $string['addentry'] = 'Add a new entry';
+$string['addsingleentry'] = 'Add entry';
 $string['addingcomment'] = 'Add a comment';
 $string['alias'] = 'Keyword';
 $string['aliases'] = 'Keyword(s)';
@@ -86,6 +88,7 @@ $string['commentdeleted'] = 'The comment has been deleted.';
 $string['comments'] = 'Comments';
 $string['commentson'] = 'Comments on';
 $string['commentupdated'] = 'The comment has been updated.';
+$string['completiondetail:entries'] = 'Make entries: {$a}';
 $string['completionentries'] = 'Student must create entries:';
 $string['completionentriesdesc'] = 'Student must create at least {$a} entry/entries';
 $string['completionentriesgroup'] = 'Require entries';
@@ -105,6 +108,7 @@ $string['defaultsortorder'] = 'Default sort order';
 $string['definition'] = 'Definition';
 $string['definitions'] = 'Definitions';
 $string['deleteentry'] = 'Delete entry';
+$string['deletecategory'] = 'Delete category';
 $string['deleteentrya'] = 'Delete entry: {$a}';
 $string['deletenotenrolled'] = 'Delete entries by users not enrolled';
 $string['deletingcomment'] = 'Deleting comment';
@@ -138,6 +142,7 @@ $string['duplicateentry'] = 'Duplicate entry';
 $string['editalways'] = 'Always allow editing';
 $string['editalways_help'] = 'This setting specifies whether entries are always editable or whether students can only edit their entries during a configured editing time (usually 30 minutes).';
 $string['editcategories'] = 'Edit categories';
+$string['editcategory'] = 'Edit category';
 $string['editentry'] = 'Edit entry';
 $string['editentrya'] = 'Edit entry: {$a}';
 $string['editingcomment'] = 'Editing comment';
@@ -176,6 +181,7 @@ $string['explainalphabet'] = 'Browse the glossary using this index';
 $string['explainexport'] = 'Click on the button below to export glossary entries.<br />You can import it anytime you wish in this or other course.<p>Please note that attachments (e.g. images) and authors are not exported.</p>';
 $string['explainimport'] = 'You must specify the file to import and define the criteria of the process.<p>Submit your request and review the results.</p>';
 $string['explainspecial'] = 'Shows entries that do not begin with a letter';
+$string['export'] = 'Export';
 $string['exportedentry'] = 'Exported entry';
 $string['exportentries'] = 'Export entries';
 $string['exportentriestoxml'] = 'Export entries to XML file';
@@ -218,8 +224,14 @@ $string['importentriesfromxml'] = 'Import entries from XML file';
 $string['includegroupbreaks'] = 'Include group breaks';
 $string['indicator:cognitivedepth'] = 'Glossary cognitive';
 $string['indicator:cognitivedepth_help'] = 'This indicator is based on the cognitive depth reached by the student in a Glossary activity.';
+$string['indicator:cognitivedepthdef'] = 'Glossary cognitive';
+$string['indicator:cognitivedepthdef_help'] = 'The participant has reached this percentage of the cognitive engagement offered by the Glossary activities during this analysis interval (Levels = No view, View, Submit)';
+$string['indicator:cognitivedepthdef_link'] = 'Learning_analytics_indicators#Cognitive_depth';
 $string['indicator:socialbreadth'] = 'Glossary social';
 $string['indicator:socialbreadth_help'] = 'This indicator is based on the social breadth reached by the student in a Glossary activity.';
+$string['indicator:socialbreadthdef'] = 'Glossary social';
+$string['indicator:socialbreadthdef_help'] = 'The participant has reached this percentage of the social engagement offered by the Glossary activities during this analysis interval (Levels = No participation, Participant alone)';
+$string['indicator:socialbreadthdef_link'] = 'Learning_analytics_indicators#Social_breadth';
 $string['isglobal'] = 'Is this glossary global?';
 $string['isglobal_help'] = 'A global glossary has entries which are linked to from throughout the site, rather than only in the course that the glossary is in. Only administrators can set a glossary as global.';
 $string['letter'] = 'letter';
@@ -260,11 +272,13 @@ $string['nopermissiontodelinglossary'] = 'You can\'t comments in this glossary!'
 $string['nopermissiontoviewresult'] = 'You can only look at results for your own entries';
 $string['notcategorised'] = 'Not categorised';
 $string['notapproved'] = 'glossary entry is not approved yet.';
+$string['entrynotapproved'] = 'Entry not approved';
 $string['numberofentries'] = 'Number of entries';
 $string['onebyline'] = '(one per line)';
 $string['page-mod-glossary-x'] = 'Any glossary module page';
 $string['page-mod-glossary-edit'] = 'Glossary add/edit entry page';
 $string['page-mod-glossary-view'] = 'View glossary edit page';
+$string['pendingapproval'] = 'Pending approval';
 $string['pluginadministration'] = 'Glossary administration';
 $string['pluginname'] = 'Glossary';
 $string['popupformat'] = 'Popup format';
@@ -280,7 +294,7 @@ $string['privacy:metadata:glossary_entries'] = 'Information about the user\'s en
 $string['privacy:metadata:glossary_entries:attachment'] = 'The attachment of the entry the user added';
 $string['privacy:metadata:glossary_entries:concept'] = 'The concept of the entry the user added';
 $string['privacy:metadata:glossary_entries:definition'] = 'The definition of the entry the user added';
-$string['privacy:metadata:glossary_entries:glossaryid'] = 'The ID of the glossary activity the user is providing entry for';
+$string['privacy:metadata:glossary_entries:glossaryid'] = 'The ID of the glossary activity';
 $string['privacy:metadata:glossary_entries:userid'] = 'The ID of the user that is adding this glossary entry';
 $string['privacy:metadata:glossary_entries:timemodified'] = 'The timestamp indicating when the glossary entry was modified by the user';
 $string['question'] = 'Question';
@@ -292,6 +306,7 @@ $string['resetglossariesall'] = 'Delete entries from all glossaries';
 $string['rssarticles'] = 'Number of RSS recent articles';
 $string['rssarticles_help'] = 'This setting specifies the number of glossary entry concepts to include in the RSS feed. Between 5 and 20 generally acceptable.';
 $string['rsssubscriberss'] = 'Display the RSS feed for \'{$a}\' concepts';
+$string['rssfeed'] = 'RSS feed';
 $string['rsstype'] = 'RSS feed for this activity';
 $string['rsstype_help'] = 'To enable the RSS feed for this activity, select either concepts with author or concepts without author to be included in the feed.';
 $string['search:activity'] = 'Glossary - activity information';
@@ -317,9 +332,11 @@ $string['totalentries'] = 'Total entries';
 $string['usedynalink'] = 'Automatically link glossary entries';
 $string['usedynalink_help'] = 'If site-wide glossary auto-linking has been enabled by an administrator and this setting is enabled, the "Add a new entry" form includes the option to automatically link the entry wherever the concept words and phrases appear throughout the rest of the course.';
 $string['visibletabs'] = 'Visible tabs';
-$string['waitingapproval'] = 'Waiting approval';
 $string['warningstudentcapost'] = '(Applies only if the glossary is not the main one)';
 $string['withauthor'] = 'Concepts with author';
 $string['withoutauthor'] = 'Concepts without author';
 $string['writtenby'] = 'by';
 $string['youarenottheauthor'] = 'You are not the author of this comment, so you are not allowed to edit it.';
+
+// Deprecated since 4.0.
+$string['waitingapproval'] = 'Waiting approval';
